module Circles where

import PointsForRendering
import Circle
import Graphics.Rendering.OpenGL
import Data.List
import FourthCircle1

circleAt x y radius = do
  translate$Vector3 x y (0::GLfloat)
  renderCircle radius
          
circleOfCenters x y r rad = do
  translate$Vector3 x y (0::GLfloat)
  renderCircle (r-rad) --Place of points of centers of the touching circle with radius rad to the circle with (x y r)

circleOfCenters2 x y r a b = do        
  translate$Vector3 x y (0::GLfloat)
  renderCircle (sqrt(((x-a)*(x-a))+((y-b)*(y-b)))) --same as previous one but with we use some (a b) to calculate rad

touchCircles x y radius a b = do
  translate$Vector3 a b (0::GLfloat)
  renderCircle (radius-(sqrt(((x-a)*(x-a))+((y-b)*(y-b))))) --two cricles touching where the first one has given (x y radius) and the second one has coordinates (they touch inward)

radComp x y radius a b = do
  radius-(sqrt(((x-a)*(x-a))+((y-b)*(y-b)))) --radius of inward touching circle with (a b) to a given circle (x y radius)
  
coordComp x y radius a b rad r = do --same as radComp but finds the coordinates
  (p,q)
  where
    p =fst (pointsOfCrossing x' y' radius' a' b' rad')
    q =snd (pointsOfCrossing x' y' radius' a' b' rad')
    x'=x
    y'=y
    a'=a
    b'=b
    radius' = radius-r  
    rad'=rad+r
    
coordComp2 x y radius a b rad r = do --same as radComp but finds the coordinates
  (p,q)
  where
    p =fst (pointsOfCrossing2 x' y' radius' a' b' rad')
    q =snd (pointsOfCrossing2 x' y' radius' a' b' rad')
    x'=x
    y'=y
    a'=a
    b'=b
    radius' = radius-r  
    rad'=rad+r    

isTouching x y radius a b r = do 
  if((r-(radius-(sqrt(((x-a)*(x-a))+((y-b)*(y-b))))))<=0.0001) then True else False

pointsOfCrossing x y rad a b r = do
  (p,q)
  where
    p=(((x+a)/2)+(((a-x)*((rad*rad)-(r*r)))/(d*2))-((2*(b-y)*k)/d))
    q=(((y+b)/2)+(((b-y)*((rad*rad)-(r*r)))/(d*2))+((2*(a-x)*k)/d))
    k=(sqrt((((rad+r)*(rad+r))-d)*(d-((rad-r)*(rad-r)))))/4
    d=((a-x)*(a-x)+(b-y)*(b-y)) --finds one of the points of crossing of 2 circles

pointsOfCrossing2 x y rad a b r = do
  (p,q)
  where
    p=(((x+a)/2)+(((a-x)*((rad*rad)-(r*r)))/(d*2))+((2*(b-y)*k)/d))
    q=(((y+b)/2)+(((b-y)*((rad*rad)-(r*r)))/(d*2))-((2*(a-x)*k)/d))
    k=(sqrt((((rad+r)*(rad+r))-d)*(d-((rad-r)*(rad-r)))))/4
    d=((a-x)*(a-x)+(b-y)*(b-y))
    
treeCirclesAll x y radius a b r = do
  translate$Vector3 p q (0::GLfloat)
  renderCircle r
  where 
    p =fst (pointsOfCrossing x' y' radius' a' b' rad')
    q =snd (pointsOfCrossing x' y' radius' a' b' rad')
    x'=x
    y'=y
    a'=a
    b'=b
    radius' = radius-r
    rad' = (radComp x y radius a b)+r
    
treeCircles x y radius a b rad r = do    
  translate$Vector3 p q (0::GLfloat)  
  renderCircle r
  where
    p =fst (pointsOfCrossing x' y' radius' a' b' rad')
    q =snd (pointsOfCrossing x' y' radius' a' b' rad')
    x'=x
    y'=y
    a'=a
    b'=b
    radius' = radius-r  
    rad'=rad+r --draws 3rd circle wich touches 2 circles one of which is incribed in the other

treeCircles2 x y radius a b rad r = do    
  translate$Vector3 p q (0::GLfloat)  
  renderCircle r
  where
    p =fst (pointsOfCrossing2 x' y' radius' a' b' rad')
    q =snd (pointsOfCrossing2 x' y' radius' a' b' rad')
    x'=x
    y'=y
    a'=a
    b'=b
    radius' = radius-r  
    rad'=rad+r

chain a1 b1 r1 a2 b2 r2 rpr i = do
  if(i>0) then (xs a1 b1 r1 a2 b2 r2 rpr):chain2  (i-1)
               else []
                    where chain2 i = do
                            (ys a1 b1 r1 a2 b2 r2 rpr):chain a1 b1 r1 a2 b2 r2 (ys a1 b1 r1 a2 b2 r2 rpr) (i-1)
 

xs a1 b1 r1 a2 b2 r2 rpr = do  (fourthcirclerad a1 b1 r1 a2 b2 r2 (fst(coordComp a1 b1 r1 a2 b2 r2 rpr)) (snd(coordComp a1 b1 r1 a2 b2 r2 rpr)) rpr )            
ys a1 b1 r1 a2 b2 r2 rpr = do                             
  (fourthcirclerad a1 b1 r1 a2 b2 r2 (fst(coordComp2 a1 b1 r1 a2 b2 r2 rpr)) (snd(coordComp2 a1 b1 r1 a2 b2 r2 rpr)) rpr )

kur a = do
  a/(1-a)

fourCirclesCoord a1 b1 r1 a2 b2 r2 a3 b3 r3 = do
  treeCircles2 a1 b1 r1 a2 b2 r2 r
  where
    r = ( fourthcirclerad a1 b1 r1 a2 b2 r2 a3 b3 r3) -- gives the fourth touching circle to 3 others
    
fourCircles a1 b1 r1 a2 b2 r2 rpr = do
  treeCircles a1 b1 r1 a2 b2 r2 r
  where
    r = (fourthcirclerad a1 b1 r1 a2 b2 r2 (fst(coordComp a1 b1 r1 a2 b2 r2 rpr)) (snd(coordComp a1 b1 r1 a2 b2 r2 rpr)) rpr )

fourCircles2 a1 b1 r1 a2 b2 r2 rpr = do
  treeCircles2 a1 b1 r1 a2 b2 r2 r
  where
    r = (fourthcirclerad a1 b1 r1 a2 b2 r2 (fst(coordComp2 a1 b1 r1 a2 b2 r2 rpr)) (snd(coordComp2 a1 b1 r1 a2 b2 r2 rpr)) rpr )
    
fourCircles3 a1 b1 r1 a2 b2 r2 rpr = do
  treeCircles2 a1 b1 r1 a2 b2 r2 r
  where
    r = (fourthcirclerad a1 b1 r1 a2 b2 r2 (fst(coordComp a1 b1 r1 a2 b2 r2 rpr)) (snd(coordComp a1 b1 r1 a2 b2 r2 rpr)) rpr )    

fourCircles4 a1 b1 r1 a2 b2 r2 rpr = do
  treeCircles a1 b1 r1 a2 b2 r2 r
  where
    r = (fourthcirclerad a1 b1 r1 a2 b2 r2 (fst(coordComp2 a1 b1 r1 a2 b2 r2 rpr)) (snd(coordComp2 a1 b1 r1 a2 b2 r2 rpr)) rpr )

{-main = do
  renderInWindow someCircles 
    
someCircles = do
  clearColor $= Color4 1 1 1 1
  clear [ColorBuffer]
    
  loadIdentity
  currentColor $= Color4 0 0 0 1  
  circleAt 0 0 0.45
  
  loadIdentity
  currentColor $=Color4 0 0 0 1
  circleAt 0 0.32 0.07
    
  loadIdentity
  currentColor $=Color4 0 0.5 0.5 1
  treeCircles 0 0 0.45 0 0.32 0.07 0.3
  
  loadIdentity 
  currentColor $=Color4 1 0 0 1
  fourCircles 0 0 0.45 0 0.32 0.07 0.3
  
  loadIdentity
  currentColor $= Color4 0 1 0 1
  fourCircles2 0 0 0.45 0 0.32 0.07 0.3
    
  loadIdentity
  currentColor $=Color4 0 0.2 0.1 1
  fourCircles3 0 0 0.45 0 0.32 0.07 0.13439022535754008
  
  loadIdentity
  currentColor $=Color4 0 0.5 0.1 1
  fourCircles4 0 0 0.45 0 0.32 0.07 7.149212758363639e-2
  
  loadIdentity
  currentColor $=Color4 0 0.1 1 1
  fourCircles2 0 0 0.45 0 0.32 0.07 4.5039082581777946e-2-} -- <- This draws the circles in different colors pic
